package com.gx.obe.common.widget.dialog;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.web.entity.price.BidPrice;
import com.gx.obe.web.service.BidPriceService;
import com.swtdesigner.ResourceManager;

/**
 * @Description: 报价明细界面
 * @author guoyr
 */
public class BidPriceItemDialog extends EDialog {
	
	private BidPrice bidPrice;
	private List<BidPrice> bidPriceList;
	private BidPriceService bidPriceService;
	private Text itemName_text;
	private Text itemCode_text;
	private String tenderId ;

	{
		LogUtils.logClass(BidPriceItemDialog.class);
	}
	
	/**
	 * Create the dialog.
	 * @param parentShell
	 */
	public BidPriceItemDialog(Shell parentShell, List<BidPrice> bidPriceList ,String tenderId) {
		super(parentShell);
		this.bidPriceList = bidPriceList;
		this.tenderId = tenderId;
	}

	/**
	 * Create contents of the dialog.
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.verticalSpacing = 0;
		gl_container.horizontalSpacing = 0;
		gl_container.marginHeight = 0;
		gl_container.marginWidth = 0;
		container.setLayout(gl_container);
		
		Composite composite = new Composite(container, SWT.NONE);
		GridLayout gl_composite = new GridLayout(2, false);
		gl_composite.marginHeight = 10;
		gl_composite.marginWidth = 10;
		composite.setLayout(gl_composite);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1));
		
		Label label_1 = new Label(composite, SWT.NONE);
		label_1.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		label_1.setText("名称：");
		
		itemName_text = new Text(composite, SWT.BORDER | SWT.DRAW_TAB);
		itemName_text.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		itemName_text.setTextLimit(100);
		
		Label label = new Label(composite, SWT.NONE);
		label.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false, false, 1, 1));
		label.setText("编码：");
		
		itemCode_text = new Text(composite, SWT.BORDER | SWT.DRAW_TAB);
		itemCode_text.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		itemCode_text.setTextLimit(100);
		
		ESepator label_2 = new ESepator(container, SWT.SEPARATOR | SWT.HORIZONTAL);
		label_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		label_2.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		Composite composite_1 = new Composite(container, SWT.NONE);
		composite_1.setBackgroundImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/fill_bg.png"));
		composite_1.setLayout(new GridLayout(2, false));
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		EButton save_btn = new EButton(composite_1, SWT.NONE);
		save_btn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/save.png"));
		GridData gd_completeBtn = new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1);
		save_btn.setLayoutData(gd_completeBtn);
		save_btn.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				saveAction();
			}
		});
		save_btn.setText(Messages.Button_ok);
		
		EButton cancel_btn = new EButton(composite_1, SWT.NONE);
		cancel_btn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/close.png"));
		cancel_btn.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				cancelAction();
			}
		});
		GridData gd_cancelBtn = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
		cancel_btn.setLayoutData(gd_cancelBtn);
		cancel_btn.setText(Messages.Button_close);
		
		init();
		return container;
	}
	
	/**
	 * @Description: 初始化界面
	 * @author guoyr
	 */
	private void init(){
		bidPriceService = new BidPriceService();
		if(null == bidPrice){
			bidPrice = new BidPrice();
		}
		itemName_text.setText(ObjectUtils.getObjString(bidPrice.getBidPriceName()));
		itemCode_text.setText(ObjectUtils.getObjString(bidPrice.getBidPriceCode()));
	}
	
	/**
	 * @Description: 保存
	 * @author guoyr
	 */
	private void saveAction(){
		if(itemName_text.getText().trim().length() == 0){
			MessageDialog.openInformation(getShell(), "名称不能为空！");
			itemName_text.setFocus();
			return;
		}
		
		boolean codeIsExist = false;
		boolean nameIsExist = false;
		if(null != bidPriceList){
			for(BidPrice item : bidPriceList){
				if(null != bidPrice.getId() && item.getId().equals(bidPrice.getId())){
					continue;
				}
				if(null != item.getBidPriceCode() && itemCode_text.getText().trim().length() > 0 && item.getBidPriceCode().equals(itemCode_text.getText().trim())){
					codeIsExist = true;
					break;
				}
				if(item.getBidPriceName().equals(itemName_text.getText().trim())){
					nameIsExist = true;
					break;
				}
			}
			if(nameIsExist){
				MessageDialog.openInformation(getShell(), "该名称" + Messages.Tips_AlreadyExistsPleaseRetype);
				itemName_text.forceFocus();
				return;
				
			}
			if(codeIsExist){
				MessageDialog.openInformation(getShell(), "该编码" + Messages.Tips_AlreadyExistsPleaseRetype);
				itemCode_text.forceFocus();
				return;
				
			}
		}
		if(null == bidPrice.getId()){
			bidPrice.setId(UuidUtils.getUUID());
			bidPrice.setTenderId(tenderId);
 			Object maxSortNo = (Integer)bidPriceService.getMaxBidPriceSortNo(tenderId);
 			bidPrice.setSortNo(null == maxSortNo ? 1 : ((Integer)maxSortNo) + 1);
		}
		bidPrice.setBidPriceCode(itemCode_text.getText().trim());
		bidPrice.setBidPriceName(itemName_text.getText().trim());
		bidPriceService.saveOrUpdate(bidPrice);
		cancelAction();
	}
	
	/**
	 * @Description: 取消
	 * @author guoyr
	 */
	private void cancelAction(){
		getShell().close();
	}
	
	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(291, 179);
	}
}
