package com.gx.obe.common.widget.dialog;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.business.entity.EditPriceItem;
import com.gx.obe.components.core.enumeration.BidPriceEnum;
import com.gx.obe.util.utils.ObjectUtils;
import org.eclipse.swt.core.utils.WidgetUtils;

/**
 * @Description: 修正报价项
 * @author chenxw
 */
public class EditPriceItemDialog extends EDialog {
	
	private Text itemNameText;
	private Composite editTypeComposite;
	
	private final EditPriceItem editPriceItem;
	
	private boolean isConfirm = false;
	
	private final String EDIT_TYPE = "editType";
	
	/**
	 * Create the dialog.
	 * 
	 * @param parentShell
	 */
	public EditPriceItemDialog(Shell parentShell, EditPriceItem editPriceItem) {
		super(parentShell);
		this.editPriceItem = editPriceItem;
	}
	
	/**
	 * Create contents of the dialog.
	 * 
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout(1, false));
		
		Composite composite_2 = new Composite(container, SWT.NONE);
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		composite_2.setLayout(new GridLayout(1, false));
		
		Composite composite = new Composite(composite_2, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, true, 1, 1));
		composite.setLayout(new GridLayout(2, false));
		
		Label label = new Label(composite, SWT.NONE);
		label.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		label.setText("修正报价名称：");
		
		itemNameText = new Text(composite, SWT.BORDER);
		itemNameText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Label label_1 = new Label(composite, SWT.NONE);
		label_1.setText("修正报价类型：");
		
		editTypeComposite = new Composite(composite, SWT.NONE);
		editTypeComposite.setLayout(new GridLayout(2, false));
		editTypeComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Button button = new Button(editTypeComposite, SWT.RADIO);
		button.setText(BidPriceEnum.getEditTypeString(BidPriceEnum.EDIT_FLOAT));
		button.setData(EDIT_TYPE, BidPriceEnum.EDIT_FLOAT);
		
		Button button_1 = new Button(editTypeComposite, SWT.RADIO);
		button_1.setText(BidPriceEnum.getEditTypeString(BidPriceEnum.EDIT_ADD_SUB));
		button_1.setData(EDIT_TYPE, BidPriceEnum.EDIT_ADD_SUB);
		
		Composite composite_1 = new Composite(container, SWT.NONE);
		composite_1.setLayout(new GridLayout(1, false));
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		EButton confirmButton = new EButton(composite_1, SWT.NONE);
		confirmButton.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				confirmAction();
			}
		});
		GridData gd_confirmButton = new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1);
		gd_confirmButton.widthHint = 77;
		confirmButton.setLayoutData(gd_confirmButton);
		confirmButton.setText("确    认");
		
		init();
		
		return container;
	}
	
	/**
	 * @Description: 初始化
	 * @author chenxw
	 */
	private void init() {
		itemNameText.setText(ObjectUtils.getObjString(editPriceItem.getItemName()));
		WidgetUtils.setRadioChecked(editTypeComposite, editPriceItem.getEditType(), EDIT_TYPE);
	}
	
	/**
	 * @Description: 确认事件
	 * @author chenxw
	 */
	private void confirmAction() {
		String itemName = itemNameText.getText().trim();
		if (itemName.isEmpty()) {
			MessageDialog.openWarning(getShell(), "修正报价名称不能为空！");
			itemNameText.setFocus();
			return;
		}
		Object editType = WidgetUtils.getRadioVal(editTypeComposite, EDIT_TYPE);
		if (null == editType) {
			MessageDialog.openWarning(getShell(), "需勾选修正报价类型！");
			return;
		}
		isConfirm = true;
		editPriceItem.setItemName(itemName);
		editPriceItem.setEditType(ObjectUtils.getObjString(editType.toString()));
		close();
	}
	
	public boolean isConfirm() {
		return isConfirm;
	}
	
	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(300, 200);
	}
}
