package com.gx.obe.common.widget.dialog;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.CallBack;
import org.eclipse.swt.core.ImageResolver;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.listener.ConfirmListener;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.LogUtils;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 
 * @author guoyr
 */
public class MessageViewDialog extends EDialog {
	
	private CallBack callBack;
	private ImageResolver imageResolver = new ImageResolver();
	private ConfirmListener confirmListener;
	private ProgressBar downloadProgressBar;
	private Composite downloadInfoComposite;
	private Text text;
	private StringBuilder message = new StringBuilder();
	private CloseMessage closeMessage;
	private int setMaximum;
	private Runnable progressRunnable;
	private int selection;
	private Composite composite_3;
	private EButton cancel_btn;
	private String clickStatus = null;
	public static final int CONFIRM_YES = 1;
	public static final int CONFIRM_NO = -1;
	public static final String CANCEL = "CANCEL";
	private List<ButtonPo> buttonPoList;
	private Map<String, EButton> operatorMap;
	
	{
		LogUtils.logClass(MessageViewDialog.class);
	}
	
	/**
	 * Create the dialog.
	 * @param parentShell
	 */
	public MessageViewDialog(Shell parentShell, int setMaximum) {
		this(parentShell, setMaximum, null);
	}
	/**
	 * @wbp.parser.constructor
	 */
	public MessageViewDialog(Shell parentShell, int setMaximum, CloseMessage closeMessage) {
		super(parentShell);
		this.closeMessage = closeMessage;
		this.setMaximum = setMaximum;
	}
	
	/**
	 * Create contents of the dialog.
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				imageResolver.disposite();
			}
		});
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.verticalSpacing = 0;
		gl_container.horizontalSpacing = 0;
		gl_container.marginHeight = 0;
		gl_container.marginWidth = 0;
		container.setLayout(gl_container);
		
		downloadInfoComposite = new Composite(container, SWT.NONE);
		downloadInfoComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		GridLayout gl_downloadInfoComposite = new GridLayout(1, false);
		downloadInfoComposite.setLayout(gl_downloadInfoComposite);
		
		text = new Text(downloadInfoComposite, SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL);
		text.setFont(SWTResourceManager.getFont("宋体", 10, SWT.NORMAL));
		text.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		ESepator label = new ESepator(container, SWT.SEPARATOR | SWT.HORIZONTAL);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		label.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		
		composite_3 = new Composite(container, SWT.NONE);
		composite_3.setBackgroundMode(SWT.INHERIT_FORCE);
		composite_3.setBackgroundImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/fill_bg.png"));
		GridLayout gl_composite_3 = new GridLayout(2, false);
		composite_3.setLayout(gl_composite_3);
		composite_3.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		if(setMaximum > 0){
			downloadProgressBar = new ProgressBar(composite_3, SWT.SMOOTH );
			downloadProgressBar.setMaximum(setMaximum);
		}else {
			downloadProgressBar = new ProgressBar(composite_3, SWT.SMOOTH | SWT.INDETERMINATE);
		}
		downloadProgressBar.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));

		init();
		return container;
	}
	
	/**
	 * @Description: 初始化
	 * @author guoyr
	 */
	private void init(){
		downloadProgressBar.setVisible(true);	
		progressRunnable = new Runnable() {
			public void run() {
				if(null != downloadProgressBar && !downloadProgressBar.isDisposed()){
					downloadProgressBar.setSelection(selection);
				}
			}
		};
		
		if(null != buttonPoList && buttonPoList.size() > 0){
			((GridLayout)composite_3.getLayout()).numColumns = buttonPoList.size() + 1;
			operatorMap = new HashMap<String, EButton>();
			if(null != cancel_btn && !cancel_btn.isDisposed()){
				cancel_btn.dispose();
			}
			for(ButtonPo ButtonPo : buttonPoList){
				int minimumWidth = ButtonPo.minimumWidth;
				String btnText = ButtonPo.btnText;
				final String key = ButtonPo.key;
				final EButton opButton = new EButton(composite_3, SWT.NONE);
				opButton.setEnabled(ButtonPo.enable);
				GridData gd_button  = null;
//				if(buttonPoList.size() == 1){
					gd_button = new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1);
//				}else {
//					gd_button = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
//				}
				gd_button = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
				if(minimumWidth > 0){
					if(btnText.length() < 5){
						gd_button.widthHint = minimumWidth;
					}
					gd_button.minimumWidth = minimumWidth;
				}
				opButton.setForeground(SWTResourceManager.getColor(80, 80, 80));
				opButton.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
				opButton.setLayoutData(gd_button);
				opButton.setText(btnText);
				opButton.setData("key", key);
				opButton.setLeftMargin(5);
//				opButton.setTopMargin(5);
//				opButton.setBottomMargin(5);
				opButton.setRightMargin(5);
				opButton.setAlpha(30);
				opButton.addMouseListener(new MouseAdapter() {
					public void mouseUp(MouseEvent e) {
						if(opButton.getEnabled()){
							clickStatus = null != key ? key : CANCEL;
							if(null != confirmListener){
								confirmListener.click(clickStatus);
							}
						}
					}
				});
				operatorMap.put(key, opButton);
			}
		}else {
			EButton cancel_btn = new EButton(composite_3, SWT.NONE);
			cancel_btn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/close.png"));
			cancel_btn.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseUp(MouseEvent e) {
					cancelAction();
				}
			});
			GridData gd_button_1 = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
			gd_button_1.widthHint = 90;
			cancel_btn.setLayoutData(gd_button_1);
			cancel_btn.setText(Messages.Button_close);
		}
	}
	
	public void addConfirmListener(ConfirmListener confirmListener){
		this.confirmListener = confirmListener;
	}
	
	public void addCancel(String btnText, boolean enable, Color color){
		addButton(btnText, null, enable, null, 90);
	}
	public void addButton(String btnText, boolean enable){
		addButton(btnText, null, enable, null, 90);
	}
	public void addButton(String btnText, String key, boolean enable){
		addButton(btnText, key, enable, null, 90);
	}
	public void addButton(String btnText, String key, boolean enable, Color bgColor){
		addButton(btnText, key, enable, bgColor, 90);
	}
	class ButtonPo{
		private String btnText;
		private String key;
		private boolean enable;
		private Color bgColr;
		private int minimumWidth;
		public ButtonPo(String btnText, String key, boolean enable, Color bgColr, int minimumWidth){
			this.btnText = btnText;
			this.key = key;
			this.enable = enable;
			this.bgColr = bgColr;
			this.minimumWidth = minimumWidth;
		}
	}
	public void addButton(String btnText, final String key, boolean enable, Color bgColr, int minimumWidth){
		if(null == buttonPoList){
			buttonPoList = new ArrayList<ButtonPo>();
		}
		buttonPoList.add(new ButtonPo(btnText, key, enable, bgColr, minimumWidth));
	}
	
	public EButton getButton(String key){
		if(null != key && null != operatorMap){
			
			return operatorMap.get(key);
		}
		return null;
	}
	public void setEnable(final String key, final boolean enable){
		if(null != key && null != operatorMap){
			Display.getDefault().syncExec(new Runnable(){
				public void run(){
					EButton operatorBtn = operatorMap.get(key);
					if(null != operatorBtn && !operatorBtn.isDisposed()){
						operatorBtn.setEnabled(enable);
					}
				}
			});
		}
	}
	public void showMessage(final String msg){
		message.append(msg).append("\r\n");
		Display.getDefault().syncExec(new Runnable() {
			public void run() {
				if(null != text && !text.isDisposed()){
					downloadProgressBar.setVisible(true);
					text.setText(message.toString());
					text.setSelection(message.length());
				}
			}
		});
	}

	public void finish(){
		Display.getDefault().syncExec(new Runnable() {
			public void run() {
				if(null != downloadProgressBar && !downloadProgressBar.isDisposed()){
					downloadProgressBar.setVisible(false);
				}
			}
		});
	}
	public void showProgress(final int selection){
		this.selection = selection;
		Display.getDefault().syncExec(progressRunnable);
	}
	
	public void setMaximum(int setMaximum){
		this.setMaximum = setMaximum;
		if(null != downloadProgressBar){
			downloadProgressBar.setMaximum(setMaximum);
		}
	}
	/**
	 * @Description: 取消
	 * @author guoyr
	 */
	private void cancelAction(){
		close();
	}

	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(498, 248);
	}
	public interface CloseMessage{
		public void close();
	}

	public CallBack getCallBack() {
		
		if(null == callBack){
			callBack = new CallBack(){
				@Override
				public void callBackError(final String error) {
					Display.getDefault().syncExec(new Runnable() {
						public void run() {
							if(null != getShell() && !getShell().isDisposed()){
								MessageDialog.openError(getShell(), error);
							}
						}
					});
				}
				
				@Override
				public void after() {
					if(setMaximum <= 0){
						Display.getDefault().syncExec(new Runnable() {
							public void run() {
								if(null != downloadProgressBar && !downloadProgressBar.isDisposed()){
									downloadProgressBar.setVisible(false);
								}
							}
						});
					}
				}
				
				@Override
				public void callBackString(String msg) {
					showMessage(msg);
				}

				@Override
				public void callBackInt(final int intValue) {
					Display.getDefault().syncExec(new Runnable() {
						public void run() {
							if(null != downloadProgressBar && !downloadProgressBar.isDisposed()){
								downloadProgressBar.setSelection(intValue);;
							}
						}
					});
				}

			};
		}
		return callBack;
	}

	public void setCallBack(CallBack callBack) {
		this.callBack = callBack;
	}
	@Override
	public boolean close() {
		if(null != closeMessage){
			closeMessage.close();
		}
		return super.close();
	}
	
}
