package com.gx.obe.common.widget.dialog;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.ImageResolver;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.utils.GridUtils;
import org.eclipse.swt.core.utils.WidgetUtils;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.widget.listener.SelectFactorListener;
import com.gx.obe.components.core.Constants;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.swtdesigner.ResourceManager;

/**
 * @Description: 评审指标选择
 * @author guoyr
 */
public class SelectFactorDialog extends EDialog {
	
	private Composite contentComposite;
	private EButton saveBtn;
	private List<EvaluationFactor> factorList;
	private List<EvaluationFactor> selectedFactorList;
	private List<String> selectedFactorIdList;
	private Grid grid;
	private SelectFactorListener listener;
	private boolean multy = true;
	private Composite buttonComposite;
	private GridColumn gridColumn;
	private Image menuOpenImage;
	private Image menuImage;
	private ImageResolver imageResolver = new ImageResolver();
	{
		LogUtils.logClass(SelectFactorDialog.class);
	}
	/**
	 * Create the dialog.
	 * @param parentShell
	 * @wbp.parser.constructor
	 */
	public SelectFactorDialog(Shell parentShell, List<EvaluationFactor> factorList, List<EvaluationFactor> selectedFactorList, boolean multy, SelectFactorListener listener) {
		super(parentShell);
		setTitle(MessageFormat.format(Messages.Button_selectFormat, Messages.Factor_factor));
		this.factorList = factorList;
		this.selectedFactorList = selectedFactorList;
		this.listener = listener;
		this.multy = multy;
	}

	/**
	 * Create contents of the dialog.
	 * @param parent
	 */
	
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.verticalSpacing = 0;
		gl_container.horizontalSpacing = 0;
		gl_container.marginHeight = 0;
		gl_container.marginWidth = 0;
		container.setLayout(gl_container);
		
		contentComposite = new Composite(container, SWT.NONE);
//		contentComposite.setBackground(SWTResourceManager.getColor(0, 102, 255));
		GridLayout gl_contentComposite = new GridLayout(1, false);
		gl_contentComposite.verticalSpacing = 0;
		gl_contentComposite.horizontalSpacing = 0;
		gl_contentComposite.marginHeight = 0;
		gl_contentComposite.marginWidth = 0;
		contentComposite.setLayout(gl_contentComposite);
		contentComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		buttonComposite = new Composite(contentComposite, SWT.NONE);
		buttonComposite.setLayout(new GridLayout(2, false));
		buttonComposite.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		
		EButton allSelect_button = new EButton(buttonComposite, SWT.NONE);
		allSelect_button.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				selectAllHistoryResultAction(true);
			}
		});
		GridData gd_allSelect_button = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_allSelect_button.widthHint = 60;
		allSelect_button.setLayoutData(gd_allSelect_button);
		allSelect_button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/select_all.png"));
		allSelect_button.setText(Messages.Button_selectAll);
		
		EButton allDesSelect_button = new EButton(buttonComposite, SWT.NONE);
		GridData gd_allDesSelect_button = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_allDesSelect_button.widthHint = 60;
		allDesSelect_button.setLayoutData(gd_allDesSelect_button);
		allDesSelect_button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/disselect_all.png"));
		allDesSelect_button.setText(Messages.Button_selectInvert);
		allDesSelect_button.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				selectAllHistoryResultAction(false);
			}
		});
		
		grid = new Grid(contentComposite, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		grid.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		grid.setAutoHeight(true);
		grid.setHeaderVisible(true);
		grid.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				gridSelection();
			}
		});
		gridColumn = new GridColumn(grid, multy ? SWT.CHECK : SWT.NONE);
		gridColumn.setWidth(200);
		gridColumn.setWordWrap(true);
		gridColumn.setText(MessageFormat.format(Messages.Title_name, Messages.Factor_factor));
		
		ESepator label = new ESepator(container, SWT.SEPARATOR | SWT.HORIZONTAL);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		label.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		Composite operateComposite = new Composite(container, SWT.NONE);
		operateComposite.setBackgroundMode(SWT.INHERIT_FORCE);
		operateComposite.setBackgroundImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/fill_bg.png"));
		GridLayout gl_operateComposite = new GridLayout(2, false);
		operateComposite.setLayout(gl_operateComposite);
		operateComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		saveBtn = new EButton(operateComposite, SWT.NONE);
		saveBtn.addMouseListener(new MouseAdapter() {
			
			public void mouseUp(MouseEvent e) {
				completeAction();
			}
		});
		GridData gd_completeBtn = new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1);
		saveBtn.setLayoutData(gd_completeBtn);
		saveBtn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/ok.png"));
		saveBtn.setText(Messages.Button_ok);
		saveBtn.setEnabled(false);
		
		EButton cancel_btn = new EButton(operateComposite, SWT.NONE);
		cancel_btn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/close.png"));
		cancel_btn.addMouseListener(new MouseAdapter() {
			
			public void mouseUp(MouseEvent e) {
				cancelAction();
			}
		});
		GridData gd_cancelBtn = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
		cancel_btn.setLayoutData(gd_cancelBtn);
		cancel_btn.setText(Messages.Button_close);
		init();
		return container;
	}
	
	/**
	 * @Description: 初始化界面
	 */
	private void init(){
		if(!multy) {
			WidgetUtils.setControlVisible(buttonComposite, false);
		}
		grid.setItemHeight(Constants.ROW_HIGHT);
		grid.setLinesVisible(false);
//		factors_grid.setAutoHeight(true);
		gridColumn.setShowTreeTogglePic(true);
//		gridColumn.setWordWrap(true);
		gridColumn.setTreeTogglePic(ImageResolver.RESOLVER_INSTANCE.loadImage(IconsActivator.PLUGIN_ID,"icons/tree_expend.png"), ImageResolver.RESOLVER_INSTANCE.loadImage(IconsActivator.PLUGIN_ID,"icons/tree_close.png"));
		gridColumn.setTree(true);
		menuOpenImage = imageResolver.loadImage(IconsActivator.PLUGIN_ID,"icons/tree_parent.png");
		menuImage = imageResolver.loadImage(IconsActivator.PLUGIN_ID,"icons/tree_item.png");
		selectedFactorIdList = new ArrayList<String>();
		if(null != selectedFactorList && selectedFactorList.size() > 0){
			for(EvaluationFactor factor : selectedFactorList){
				selectedFactorIdList.add(factor.getId());
			}
		}
		if(null != factorList && factorList.size() > 0){
			for(EvaluationFactor factor : factorList){
				GridItem gridItem = new GridItem(grid, SWT.NONE);
				initChildGridItem(factor, gridItem);
			}
			saveBtn.setEnabled(true);
			
			if(grid.getSelectionIndex() > 0) {
				Rectangle rectangle = grid.getBounds();
				grid.scroll(0, 0, 0, 0, rectangle.width, rectangle.height, true);
				grid.getVerticalBar().setSelection(grid.getSelectionIndex() -5);
				grid.getVerticalBar().notifyListeners(SWT.Selection, null);
			}
			}
	}
	
	/**
	 * @Description: 递归实现指标树
	 * @author guoyr
	 * @param factor
	 * @param parentGridItem
	 * @param selectId 
	 */
	private void initChildGridItem(EvaluationFactor factor, GridItem parentGridItem){
		if(null != factor.getChildFactorList() && factor.getChildFactorList().size() > 0){
			for(EvaluationFactor childFactor : factor.getChildFactorList()){
				GridItem gridItem = new GridItem(parentGridItem, SWT.NONE);
				initChildGridItem(childFactor, gridItem);
			}
			parentGridItem.setExpanded(true);
			parentGridItem.setImage(0, menuOpenImage);
		}else {
			parentGridItem.setImage(0, menuImage);
		}
		parentGridItem.setText(0, factor.getFactorName());
		parentGridItem.setData("evaluationFactor", factor);
		if(multy) {
			if(selectedFactorIdList.remove(factor.getId())) {
				parentGridItem.setChecked(true);
			}
		}else {
			if(selectedFactorIdList.remove(factor.getId())) {
				grid.setSelection(new GridItem[]{parentGridItem});
			}
		}
	}
	
	/**
	 * @Description: 保存
	 */
	private void completeAction(){
		if(saveBtn.getEnabled()){
			if(multy) {
				GridItem[] items = grid.getItems();
				List<EvaluationFactor> selectEvaluationFactorList = new ArrayList<EvaluationFactor>();
				for(GridItem item : items){
					if(item.getChecked()){
						EvaluationFactor evaluationFactor = (EvaluationFactor)item.getData("evaluationFactor");
						selectEvaluationFactorList.add(evaluationFactor);
					}
				}
				if(selectEvaluationFactorList.size() == 0) {
					MessageDialog.openShortMessage(MessageFormat.format(Messages.Tips_pleaseSelectLeastOne, Messages.Factor_factor));
					return;
				}
				listener.setEvaluationFactorList(selectEvaluationFactorList);
			}else {
				if(grid.getSelectionCount() == 0) {
					MessageDialog.openShortMessage(MessageFormat.format(Messages.Tips_pleaseSelect, Messages.Factor_factor));
					return;
				}
				EvaluationFactor evaluationFactor = (EvaluationFactor)grid.getSelection()[0].getData("evaluationFactor");
				listener.setEvaluationFactor(evaluationFactor);
			}
			getShell().close();
		}
	}
	/**
	 * @Description: 全选、反选
	 * @author guoyr
	 * @param selectAll
	 */
	private void selectAllHistoryResultAction(boolean selectAll){
		GridUtils.checkAll(grid, 0, selectAll);
		saveBtn.setEnabled(selectAll);
	}
	
	/**
	 * @Description: 表格选取事件
	 * @author guoyr 
	 */
	private void gridSelection(){
	}
	
	@Override
	public boolean close() {
		imageResolver.disposite();
		return super.close();
	}

	/**
	 * @Description: 取消
	 */
	private void cancelAction(){
		getShell().close();
	}
	/**
	 * Return the initial size of the dialog.
	 */
	
	protected Point getInitialSize() {
		return new Point(500, 400);
	}
}
