package com.gx.obe.common.widget.dialog;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.business.entity.EditPriceItem;
import com.gx.obe.common.widget.composite.SetItemListComposite;
import com.gx.obe.common.widget.listener.SetItemListListener;
import com.gx.obe.components.core.enumeration.BidPriceEnum;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.EditPriceItemService;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 设置修正报价项
 * @author chenxw
 */
public class SetEditPriceItemDialog extends EDialog implements SetItemListListener {
	
	private EditPriceItemService editPriceItemService ;
	
	private SetItemListComposite itemListComposite;
	private EButton saveButton;
	
	private final TenderProjectEntity tenderProjectEntity;
	
	private List<EditPriceItem> editPriceItemList;
	private List<EditPriceItem> removeList = new ArrayList<EditPriceItem>();
	
	/**
	 * Create the dialog.
	 * 
	 * @param parentShell
	 */
	public SetEditPriceItemDialog(Shell parentShell, TenderProjectEntity tenderProjectEntity) {
		super(parentShell);
		setTitle("设置修正报价项");
		
		this.tenderProjectEntity = tenderProjectEntity;
	}
	
	/**
	 * Create contents of the dialog.
	 * 
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.verticalSpacing = 0;
		container.setLayout(gl_container);
		
		itemListComposite = new SetItemListComposite(container, SWT.NONE);
		itemListComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		GridLayout gl_itemListComposite = new GridLayout(1, false);
		gl_itemListComposite.verticalSpacing = 1;
		gl_itemListComposite.marginHeight = 1;
		gl_itemListComposite.marginWidth = 1;
		itemListComposite.setLayout(gl_itemListComposite);
		
		Composite buttonComposite = new Composite(container, SWT.NONE);
		buttonComposite.setLayout(new GridLayout(1, false));
		buttonComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		saveButton = new EButton(buttonComposite, EButton.CSS_GREEN);
		saveButton.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				saveAction();
			}
		});
		saveButton.setFont(SWTResourceManager.getFont("微软雅黑", 11, SWT.BOLD));
		GridData gd_saveButton = new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1);
		gd_saveButton.heightHint = 30;
		gd_saveButton.widthHint = 90;
		saveButton.setLayoutData(gd_saveButton);
		saveButton.setText("保    存");
		
		init();
		return container;
	}
	
	/**
	 * @Description: 初始化
	 * @author chenxw
	 */
	private void init() {
		editPriceItemService = new EditPriceItemService();
		editPriceItemList = editPriceItemService.getEditPriceItemList(tenderProjectEntity.getId());
		
		itemListComposite.addSetItemListListener(this);
		itemListComposite.init();
	}
	
	/**
	 * @Description: 保存事件
	 * @author chenxw
	 */
	private void saveAction() {
		for (int i = 0; i < editPriceItemList.size(); i++) {
			EditPriceItem editPriceItem = editPriceItemList.get(i);
			if (!BidPriceEnum.DEVIATE_CALUSE.equals(editPriceItem.getItemCode())) editPriceItem.setSortNo(i + 1);
		}
		editPriceItemService.saveEditPriceItemList(editPriceItemList, removeList);
		close();
	}
	
	/**
	 * @Description: 设置表格值
	 * @author chenxw
	 * @param editPriceItem
	 * @param gridItem
	 */
	private void setGridItemContent(EditPriceItem editPriceItem, GridItem gridItem) {
		gridItem.setText(0, editPriceItem.getItemName());
		gridItem.setText(1, BidPriceEnum.getEditTypeString(editPriceItem.getEditType()));
	}
	
	/**
	 * @Description: 获取项名称
	 * @author chenxw
	 * @param selectIndex
	 * @return
	 */
	public String getItemName(int selectionIndex) {
		return editPriceItemList.get(selectionIndex).getItemName();
	}
	
	/**
	 * @Description: 是否可以编辑
	 * @author chenxw
	 * @param selectionIndex
	 * @return
	 */
	public boolean getEditEnabled(int selectionIndex) {
		if (null != editPriceItemList.get(selectionIndex).getId()) {
			return false;
		}
		if (BidPriceEnum.DEVIATE_CALUSE.equals(editPriceItemList.get(selectionIndex).getItemCode())) {
			return false;
		}
		return true;
	}
	
	/**
	 * @Description: 是可以否删除
	 * @author chenxw
	 * @param selectionIndex
	 * @return
	 */
	public boolean getDeleteEnabled(int selectionIndex) {
		return !BidPriceEnum.DEVIATE_CALUSE.equals(editPriceItemList.get(selectionIndex).getItemCode());
	}
	
	/**
	 * @Description: 初始化表格列
	 * @author chenxw
	 * @param itemListGrid
	 */
	public void initItemListGridColumn(Grid itemListGrid) {
		GridColumn gridColumn1 = new GridColumn(itemListGrid, SWT.NONE);
		gridColumn1.setText(Messages.Name);
		gridColumn1.setWidth(300);
		
		GridColumn gridColumn2 = new GridColumn(itemListGrid, SWT.NONE);
		gridColumn2.setText("修正类型");
		gridColumn2.setWidth(200);
		gridColumn2.setAlignment(SWT.CENTER);
	}
	
	/**
	 * @Description: 初始化表格行
	 * @author chenxw
	 * @param itemListGrid
	 */
	public void initItemListGridRows(Grid itemListGrid) {
		if (CollectionUtils.isNull(editPriceItemList)) return;
		Collections.sort(editPriceItemList, new Comparator<EditPriceItem>() {
			public int compare(EditPriceItem item1, EditPriceItem item2) {
				return item1.getSortNo() - item2.getSortNo();
			}
		});
		for (EditPriceItem editPriceItem : editPriceItemList) {
			setGridItemContent(editPriceItem, new GridItem(itemListGrid, SWT.NONE));
		}
	}
	
	/**
	 * @Description: 添加事件
	 * @author chenxw
	 * @param itemListGrid
	 * @return
	 */
	public boolean addItemAction(Grid itemListGrid) {
		EditPriceItem editPriceItem = new EditPriceItem();
		editPriceItem.setTenderId(tenderProjectEntity.getId());
		
		EditPriceItemDialog dialog = new EditPriceItemDialog(getShell(), editPriceItem);
		dialog.setTitle("添加修正报价项");
		dialog.open();
		
		if (dialog.isConfirm()) {
			editPriceItemList.add(editPriceItem);
			setGridItemContent(editPriceItem, new GridItem(itemListGrid, SWT.NONE));
		}
		return dialog.isConfirm();
	}
	
	/**
	 * @Description: 编辑事件
	 * @author chenxw
	 * @param itemListGrid
	 * @param selectIndex
	 * @return
	 */
	public boolean editItemAction(Grid itemListGrid, int selectIndex) {
		EditPriceItem editPriceItem = editPriceItemList.get(selectIndex);
		
		EditPriceItemDialog dialog = new EditPriceItemDialog(getShell(), editPriceItem);
		dialog.setTitle("修改修正报价项");
		dialog.open();
		
		if (dialog.isConfirm()) {
			setGridItemContent(editPriceItem, itemListGrid.getItem(selectIndex));
		}
		return dialog.isConfirm();
	}
	
	/**
	 * @Description: 删除事件
	 * @author chenxw
	 * @param selectIndex
	 * @return
	 */
	public boolean deleteItemAction(int selectIndex) {
		boolean isConfirm = MessageDialog.openConfirm(getShell(), "删除修正报价项", "是否确认删除修正报价项？", new String[]{Messages.Button_confirm, Messages.Button_cancel});
		if (isConfirm) {
			removeList.add(editPriceItemList.get(selectIndex));
			editPriceItemList.remove(selectIndex);
		}
		return isConfirm;
	}
	
	/**
	 * Return the initial size of the dialog.
	 */
	protected Point getInitialSize() {
		return new Point(600, 400);
	}
	
}
